
using System;
using System.Collections.Concurrent;
using System.Linq;
using System.Threading.Tasks;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;

namespace WingEducationService.Common
{
    /// <summary>
    /// 直播心跳管理
    /// </summary>
    public class LiveHeartRateManager
    {
        private int _rateSeconds = 10;//秒
        private int _networkErrTimes = 3;//3次间隔无心跳，网络不佳
        private int _leaveRoomTimes = 10;//10次间隔无心跳，离开房间
        private ConcurrentDictionary<string, LiveHeartRateInfo> _clientTokens = new ConcurrentDictionary<string, LiveHeartRateInfo>();

        private Action<string, string, string> OnClientJoined;
        private Action<string, string, string> OnClientNetworkErr;
        private Action<string, string, string> OnClientLeave;

        public LiveHeartRateManager(Action<string, string, string> handleClientJoined, Action<string, string, string> handleClientNetworkErr, Action<string, string, string> handleClientLeave)
        {
            _rateSeconds = ConfigurationManager.GetParammeter<IntParameter>("Live", "HeartRateSeconds").Value;
            _networkErrTimes = ConfigurationManager.GetParammeter<IntParameter>("Live", "NetworkErrTimes").Value;
            _leaveRoomTimes = ConfigurationManager.GetParammeter<IntParameter>("Live", "LeaveRoomTimes").Value;
            OnClientJoined = handleClientJoined;
            OnClientNetworkErr = handleClientNetworkErr;
            OnClientLeave = handleClientLeave;
            StartCheckClients();
        }

        /// <summary>
        /// 加入检测集合
        /// </summary>
        /// <param name="token"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <param name="consultationCode"></param>
        public void AddOrUpdate(string token, string clientId, string name, string liveCode)
        {
            var uniqueId = GetUniqueId(token, liveCode);
            _clientTokens.AddOrUpdate(uniqueId, (k) =>
            {
                var liveHeartRateInfo = new LiveHeartRateInfo()
                {
                    Token = token, 
                    ClientId = clientId, 
                    Name = name, 
                    LiveCode = liveCode
                };
                OnClientJoined.Invoke(liveCode, liveHeartRateInfo.ClientId, name);
                return liveHeartRateInfo;
            }, (k, v) =>
            {
                v.Activate();
                OnClientJoined.Invoke(liveCode, v.ClientId, name);
                return v;
            });
        }

        /// <summary>
        /// get uniquedId
        /// </summary>
        /// <param name="token"></param>
        /// <param name="liveCode"></param>        
        /// <returns>uniqueId</returns>
        private string GetUniqueId(string token, string liveCode)
        {
            var uniqueId = $"{liveCode}_{token}";
            return uniqueId;
        }

        /// <summary>
        /// 会诊心跳有效期验证
        /// </summary>
        private void StartCheckClients()
        {
            Task.Run(async () =>
            {
                Logger.WriteLineInfo($"LiveHeartRateManager start check clients");
                try
                {
                    while (true)
                    {
                        await Task.Delay(1 * 1000 * _rateSeconds);
                        foreach (var tokenInfo in _clientTokens.Values)
                        {
                            tokenInfo.DeActivate();
                            var uniqueId = GetUniqueId(tokenInfo.Token, tokenInfo.LiveCode);
                            try
                            {
                                if (tokenInfo.LivingTimes == _networkErrTimes)
                                {
                                    OnClientNetworkErr.Invoke(tokenInfo.LiveCode, tokenInfo.ClientId, tokenInfo.Name);
                                }
                                else if (tokenInfo.LivingTimes == _leaveRoomTimes)
                                {
                                    if (!_clientTokens.Values.Any(x => x.ClientId == tokenInfo.ClientId && x.LiveCode == tokenInfo.LiveCode && x.LivingTimes < _leaveRoomTimes))
                                    {
                                        OnClientLeave.Invoke(tokenInfo.LiveCode, tokenInfo.ClientId, tokenInfo.Name);
                                    }
                                    _clientTokens.TryRemove(uniqueId, out _);
                                }
                            }
                            catch (Exception ex)
                            {
                                Logger.WriteLineWarn($"check user[{tokenInfo.Name}] token {tokenInfo.Token} err, {ex}");
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"LiveHeartRateManager err, ex:{ex}");
                }
                Logger.WriteLineInfo($"LiveHeartRateManager finished");
            });
        }

    }
}