using System.Security.Permissions;
using System.Net;
using System.Net.Cache;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Request.Courses;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request.DBRequest;
using System.Linq;
using WingInterfaceLibrary.Result.Courses;
using WingInterfaceLibrary.Request.Payment;
using WingServerCommon.Log;
using WingInterfaceLibrary.Notifications.Consultation;
using WingInterfaceLibrary.Internal.Request;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.Notifications.Education;

namespace WingEducationService.Service
{
    /// <summary>
    /// 课程服务服务
    /// </summary>
    public partial class EducationService : JsonRpcService, IEducationService
    {
        /// <summary>
        /// 申请课程
        /// </summary>
        /// <param name="request">申请课程请求</param>
        /// <returns>成功或者失败</returns>
        /// <errorCodes></errorCodes>
        public async Task<bool> ApplyCourseAsync(ApplyCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //用户权限验证
            if (string.IsNullOrEmpty(request.TeacherCode))
            {
                ThrowRpcException(CustomerRpcCode.TeacherCodeEmpty, "Teacher code  is empty");
            }
            var teacherDTO = await _userDBService.FindUserByCodeAsync(request.TeacherCode);
            if (teacherDTO == null)
            {
                ThrowRpcException(CustomerRpcCode.TeacherNoFound, "Teacher name no found");
            }
            var creatorDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (creatorDTO == null)
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User no found");
            }
            var createCourseInfoDTO = new CourseInfoDBRequest()
            {
                Name = request.Name,
                CourseIntro = request.CourseIntro,
                TeacherCode = request.TeacherCode,
                Poster = request.Poster,
                StartTime = request.StartTime,
                Duration = request.Duration,
                CourseType = request.CourseType,
                AudienceType = request.AudienceType,
                CoursewareToken = request.CoursewareToken,
                Status = CourseStatusEnum.NoApproval,
                TeacherName = teacherDTO.FullName,
                CreatorCode = userCode,
                OrganizationCode = creatorDTO.OrganizationCode,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes,
                UserGroupCodes = request.UserGroupCodes,
                Price = request.Price,
                IsAgentCourse = request.IsAgentCourse,
                BindExams = request.BindExams,
                Assistants = new List<StudentInfoDTO>(),
                Experts = new List<StudentInfoDTO>()
            };
            if (request.Assistants?.Count > 0)
            {
                foreach(var item in request.Assistants)
                {
                    var assistant = new StudentInfoDTO()
                    {
                        Code = item,
                        SignCourseStatus = StudentCourseStatusEnum.SignUp
                    };
                    createCourseInfoDTO.Assistants.Add(assistant);
                }
            }
            if (request.Experts?.Count > 0)
            {
                foreach(var item in request.Experts)
                {
                    var expert = new StudentInfoDTO()
                    {
                        Code = item,
                        SignCourseStatus = StudentCourseStatusEnum.SignUp
                    };
                    createCourseInfoDTO.Experts.Add(expert);
                }
            }
            var courseCode = await _courseDBService.CreateCourseAsync(new CreateCourseDBRequest() { Data = createCourseInfoDTO });

            if (!string.IsNullOrWhiteSpace(courseCode) && createCourseInfoDTO.Price > 0)
            {
                var createProductRequest = new CreateProductDBRequest
                {
                    Name = createCourseInfoDTO.Name,
                    HeadImage = createCourseInfoDTO.Poster,
                    Price = decimal.ToDouble(createCourseInfoDTO.Price),
                    Status = ProductStatusEnum.SoldOut,
                    ProductType = ProductTypeEnum.Course,
                    BelongerCode = createCourseInfoDTO.TeacherCode,
                    RelationInfo = courseCode,
                };
                await _paymentDBService.CreateProductAsync(createProductRequest);
            }
            return !string.IsNullOrWhiteSpace(courseCode);
        }

        /// <summary>
        /// 查询课程列表分页
        ///查询自己申请课程列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<CoursePageDTO>> FindCoursePagesAsync(FindCoursePagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            var dbRequest = new FindCoursePagesDBRequest()
            {
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                Keyword = request.Keyword,
                UserCode = userCode,
            };
            var result = await _courseDBService.FindCoursePagesAsync(dbRequest);
            var coursePages = new PageResult<CoursePageDTO>();
            if (result == null)
            {
                return coursePages;
            }
            coursePages.TotalCount = result.TotalCount;
            coursePages.PageSize = result.PageSize;
            coursePages.PageIndex = result.PageIndex;
            if (result.PageData != null && result.PageData.Count > 0)
            {
                coursePages.PageData = result.PageData.MappingTo<List<CoursePageDTO>>();
            }
            return coursePages;

        }

        /// <summary>
        /// 删除课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteCourseByCodeAsync(DeleteCourseByCodeRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            //角色权限验证
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            if (existCourse.Status == CourseStatusEnum.RejectApproval || existCourse.Status == CourseStatusEnum.HasCancelled || existCourse.Status == CourseStatusEnum.NoApproval)//已经审核通过
            {
                return await _courseDBService.DeleteCourseByCodeAsync(new DeleteCourseByCodeDBRequest() { Code = request.Code });
            }
            else
            {
                ThrowRpcException(CustomerRpcCode.NotDeleteForCourseStatus, "Not delete for course status");
            }
            return false;
        }

        /// <summary>
        /// 更新课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseAsync(UpdateCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var teacherDTO = await _userDBService.FindUserByCodeAsync(request.TeacherCode);
            if (teacherDTO == null)
            {
                ThrowRpcException(CustomerRpcCode.TeacherNoFound, "Teacher name no found");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //检查状态
            if (existCourse.Status == CourseStatusEnum.RejectApproval || existCourse.Status == CourseStatusEnum.NoApproval)
            {
                var updateCourseInfoDTO = new CourseInfoDBRequest()
                {
                    Name = request.Name,
                    CourseIntro = request.CourseIntro,
                    TeacherCode = request.TeacherCode,
                    Poster = request.Poster,
                    StartTime = request.StartTime,
                    Duration = request.Duration,
                    CourseType = request.CourseType,
                    AudienceType = request.AudienceType,
                    CoursewareToken = request.CoursewareToken,
                    TeacherName = teacherDTO.FullName,
                    CaseLabelCodes = request.CaseLabelCodes,
                    CourseLabelCodes = request.CourseLabelCodes,
                    UserGroupCodes = request.UserGroupCodes,
                    Price = request.Price,
                    IsAgentCourse = request.IsAgentCourse,
                    BindExams = request.BindExams,
                    Assistants = new List<StudentInfoDTO>(),
                    Experts = new List<StudentInfoDTO>()
                };
                if (request.Assistants?.Count > 0)
                {
                    foreach(var item in request.Assistants)
                    {
                        var assistant = new StudentInfoDTO()
                        {
                            Code = item,
                            SignCourseStatus = StudentCourseStatusEnum.SignUp
                        };
                        updateCourseInfoDTO.Assistants.Add(assistant);
                    }
                }
                if (request.Experts?.Count > 0)
                {
                    foreach(var item in request.Experts)
                    {
                        var expert = new StudentInfoDTO()
                        {
                            Code = item,
                            SignCourseStatus = StudentCourseStatusEnum.SignUp
                        };
                        updateCourseInfoDTO.Experts.Add(expert);
                    }
                }
                var updateCourseResult = await _courseDBService.UpdateCourseAsync(new UpdateCourseDBRequest() { Data = updateCourseInfoDTO, Code = code });
                if (updateCourseResult && (existCourse.Price > 0 || updateCourseInfoDTO.Price > 0))
                {
                    var updateProductRequest = new UpdateProductByRelationDBRequest
                    {
                        RelationInfo = code,
                        Name = updateCourseInfoDTO.Name,
                        HeadImage = updateCourseInfoDTO.Poster,
                        Price = decimal.ToDouble(updateCourseInfoDTO.Price),
                        Status = (existCourse.Status == CourseStatusEnum.NoApproval || existCourse.Status == CourseStatusEnum.RejectApproval) ? ProductStatusEnum.SoldOut : ProductStatusEnum.Putaway,
                        ProductType = ProductTypeEnum.Course,
                        BelongerCode = updateCourseInfoDTO.TeacherCode,
                    };
                    await _paymentDBService.UpdateProductByRelationAsync(updateProductRequest);
                }
                return updateCourseResult;
            }
            else
            {
                ThrowRpcException(CustomerRpcCode.NotUpdateForCourseStatus, "Not update for course status");
            }
            return false;
        }

        /// <summary>
        /// 根据课程编码获取课程详细信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseInfoDetailDTO> FindCourseByCodeAsync(FindCourseByCodeRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var resultData = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code });
            var student = resultData.Students.FirstOrDefault(x => x.Code == userCode);
            if (student != null)
            {
                resultData.IsPay = student.IsPay;
                //resultData.SignCourseStatus = student.SignCourseStatus;
            }
            if (resultData.Experts?.Count > 0)
            {
                foreach(var expertItem in resultData.Experts)
                {
                    var userInfo = CacheMaintenance.Instance.Get<IUserInfoManager>().Get(expertItem.Code);
                    expertItem.Name = !string.IsNullOrEmpty(userInfo.FullName) ? userInfo.FullName : userInfo.UserName;
                }
            }
            if (resultData.Assistants?.Count > 0)
            {
                foreach(var assistantItem in resultData.Assistants)
                {
                    var userInfo = CacheMaintenance.Instance.Get<IUserInfoManager>().Get(assistantItem.Code);
                    assistantItem.Name = !string.IsNullOrEmpty(userInfo.FullName) ? userInfo.FullName : userInfo.UserName;
                }
            }
            return resultData;
        }

        /// <summary>
        /// 查询自己当学生的课程列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<StudentInCoursePageDTO>> FindStudentInCoursePagesAsync(FindStudentInCoursePagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            var dbRequest = new FindStudentCoursePagesDBRequest()
            {
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                Keyword = request.Keyword,
                UserCode = userCode,
                QueryStatus = request.QueryStatus,
                StartTime = request.StartTime,
                QueryCourseSort = request.QueryCourseSort,
                QueryCourseType = request.CourseType
            };
            var result = await _courseDBService.FindStudentInCoursePagesAsync(dbRequest);
            var coursePages = new PageResult<StudentInCoursePageDTO>();
            if (result == null)
            {
                return coursePages;
            }
            coursePages.TotalCount = result.TotalCount;
            coursePages.PageSize = result.PageSize;
            coursePages.PageIndex = result.PageIndex;
            if (result.PageData != null && result.PageData.Count > 0)
            {
                coursePages.PageData = result.PageData.MappingTo<List<StudentInCoursePageDTO>>();
            }
            foreach (var course in coursePages.PageData)
            {
                var dto = result.PageData.FirstOrDefault(x => x.Code == course.Code);
                var student = dto.Students.FirstOrDefault(x => x.Code == userCode);
                if (student != null)
                {
                    course.IsPay = student.IsPay;
                    //course.SignCourseStatus = student.SignCourseStatus;
                }
            }
            return coursePages;
        }

        /// <summary>
        /// 新增试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddExaminationPaperAsync(AddExaminationPaperRequest request)
        {
            var dbRequest = new AddExaminationPaperDBRequest
            {
                Name = request.Name,
                TotalScore = request.TotalScore,
                QuestionList = request.QuestionList,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            };
            return await _educationDBService.AddExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 删除试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteExaminationPaperAsync(DeleteExaminationPaperRequest request)
        {
            var dbRequest = new DeleteExaminationPaperDBRequest
            {
                ExaminationPaperCode = request.ExaminationPaperCode
            };
            return await _educationDBService.DeleteExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 修改试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateExaminationPaperAsync(UpdateExaminationPaperRequest request)
        {
            var dbRequest = new UpdateExaminationPaperDBRequest
            {
                ExaminationPaperCode = request.ExaminationPaperCode,
                Name = request.Name,
                TotalScore = request.TotalScore,
                QuestionList = request.QuestionList,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            };
            return await _educationDBService.UpdateExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 查询试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ExaminationPaperDTO> GetExaminationPaperAsync(GetExaminationPaperRequest request)
        {
            var dbRequest = new GetExaminationPaperDBRequest
            {
                ExaminationPaperCode = request.ExaminationPaperCode
            };
            return await _educationDBService.GetExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 查询试卷分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ExaminationPaperDTO>> GetExaminationPaperPagesAsync(GetExaminationPaperPagesRequest request)
        {
            var dbRequest = new GetExaminationPaperPagesDBRequest
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            };
            return await _educationDBService.GetExaminationPaperPagesAsync(dbRequest);
        }

        /// <summary>
        /// 创建视频
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateVideoAsync(SaveVideoRequest request)
        {
            //Code验证
            if (string.IsNullOrEmpty(request.Name))
            {
                ThrowRpcException(CustomerRpcCode.VideoNameIsEmpty, "Video name is empty");
            }
            if (string.IsNullOrEmpty(request.VideoToken))
            {
                ThrowRpcException(CustomerRpcCode.VideoTokenIsEmpty, "Video token is empty");
            }
            var result = await _courseDBService.CreateVideoDBAsync(request);
            return result;
        }

        /// <summary>
        /// 查询所有视频列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<VideoInfoDTO>> FindVideosPagesAsync(FindVideoPagesRequest request)
        {
            var result = await _courseDBService.FindVideosPagesDBAsync(request);
            return result;
        }

        /// <summary>
        /// 删除视频，同步删除课程管理视频code
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteVideoByCodeAsync(DeleteVideoRequest request)
        {
            //Code验证
            if (string.IsNullOrEmpty(request.Code))
            {
                ThrowRpcException(CustomerRpcCode.VideoCodeIsEmpty, "video code is empty");
            }
            var result = await _courseDBService.DeleteVideoByCodeDBAsync(request);
            return result;
        }

        /// <summary>
        /// 购买课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<BuyCourseResult> BuyCourseAsync(BuyCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var course = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = request.CourseCode });
            if (course == null || string.IsNullOrWhiteSpace(course.Code))
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "CourseNoFound");
            }
            if (course.Status == CourseStatusEnum.NoApproval || course.Status == CourseStatusEnum.RejectApproval)
            {
                ThrowRpcException(CustomerRpcCode.CourseNotReviewed, "CourseNotReviewed");
            }
            if (course.Price <= 0)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoPayRequired, "CourseNoPayRequired");
            }
            if (request.Quantity <= 0)
            {
                ThrowRpcException(CustomerRpcCode.CourseQuantityError, "CourseQuantityError");
            }
            var product = await _paymentDBService.GetProductByRelationInfoAsync(new GetProductByRelationInfoDBRequest { RelationInfo = course.Code });
            if (product == null || string.IsNullOrWhiteSpace(product.Code) || product.Status != ProductStatusEnum.Putaway)
            {
                ThrowRpcException(CustomerRpcCode.CourseNotReviewed, "CourseNotReviewed");
            }
            var orderAmount = decimal.ToDouble(course.Price * request.Quantity);
            var createOrderRequest = new CreateOrderRequest
            {
                OrderUserCode = userCode,
                ProductCode = product.Code,
                OrderTitle = course.Name,
                OrderType = OrderTypeEnum.General,
                OrderAmount = orderAmount,
            };
            var createOrderResult = await _paymentService.CreateOrderAsync(createOrderRequest);
            var orderCode = createOrderResult.OrderCode;
            if (string.IsNullOrWhiteSpace(orderCode))
            {
                ThrowRpcException(CustomerRpcCode.CreateCourseOrderFailed, "CreateCourseOrderFailed");
            }
            var prepayResult = await _paymentService.GoToPayAsync(new GoToPayRequest
            {
                OrderCode = orderCode,
                PayType = request.PayType,
            });
            var result = new BuyCourseResult
            {
                PaymentOrderCode = orderCode,
                PayUrl = prepayResult.PayUrl,
                OrderTitle = course.Name,
                OrderAmount = orderAmount
            };
            return result;
        }

        /// <summary>
        /// 支付结果回调
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> PaymentCallbackAsync(PaymentCallbackRequest request)
        {
            if (request.PayStatus == PayStatusEnum.Paid)
            {
                var paySuccessRequest = new PaySuccessDBRequest
                {
                    PaymentOrderCode = request.PaymentOrderCode,
                };
                var paySuccessResult = await _paymentDBService.PaySuccessAsync(paySuccessRequest);
                if (paySuccessResult.Success)
                {
                    if (!string.IsNullOrWhiteSpace(paySuccessResult.OrderUserCode))
                    {
                        //课程支付成功通知 
                        var paySuccessNotification = new CourcePaySuccessNotification
                        {
                            PaymentOrderCode = request.PaymentOrderCode,
                            OrderTitle = paySuccessResult.OrderTitle,
                            OrderAmount = paySuccessResult.OrderAmount,
                            PayTime = paySuccessResult.PayTime,
                        };
                        var broadcastNotificationRequest = new BroadcastNotificationRequest
                        {
                            ClientIds = new List<string> { paySuccessResult.OrderUserCode },
                            Message = paySuccessNotification,
                            JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(paySuccessNotification),
                            NotificationType = paySuccessNotification.NotificationType,
                            TransactionType = TransactionTypeEnum.Education,
                            RelevanceCode = paySuccessResult.RelationInfo,
                            ReceiverType = ApplicantTypeEnum.Client
                        };
                        await _notificationService.BroadcastMessageAsync(broadcastNotificationRequest);
                    }
                    return true;
                }
                else
                {
                    Logger.WriteLineWarn($"EducationService PaymentCallbackAsync failed, order code:{request.PaymentOrderCode}, pay status:{request.PayStatus.ToString()}");
                }
            }
            else
            {
                Logger.WriteLineWarn($"EducationService PaymentCallbackAsync, order code:{request.PaymentOrderCode}, pay status:{request.PayStatus.ToString()}");
            }
            return await Task.FromResult(false);
        }


        /// <summary>
        /// 报名课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SignUpCourseAsync(SignUpCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //TODO 检查状态  “已审核”
            if (existCourse.Status == CourseStatusEnum.Approved)
            {
                var students = existCourse.Students ?? new List<StudentInfoDTO>();
                if (students.FirstOrDefault(c => c.Code == userCode) != null)
                {
                    ThrowRpcException(CustomerRpcCode.SignedUp, "Course signed up");
                }
                students.Add(new StudentInfoDTO()
                {
                    Code = userCode,
                    SignCourseStatus = StudentCourseStatusEnum.SignUp
                });
                return await _courseDBService.UpdateCourseByStudentsDBAsync(new UpdateCourseByStudentsDBRequest()
                {
                    CourseCode = courseCode,
                    Students = students
                });
            }
            else
            {
                ThrowRpcException(CustomerRpcCode.NotUpdateForCourseStatus, "Not update for course status");
            }
            return false;

        }

        /// <summary>
        /// 取消报名课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CancelSignUpCourseAsync(CancelSignUpCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //TODO 检查状态 “已审核”
            if (existCourse.Status == CourseStatusEnum.Approved)
            {
                var students = existCourse.Students;
                var signedUpStudent = students.FirstOrDefault(c => c.Code == userCode);
                if (signedUpStudent == null)
                {
                    ThrowRpcException(CustomerRpcCode.NoSignedUp, "Course no signed up");
                }
                students.Remove(signedUpStudent);
                return await _courseDBService.UpdateCourseByStudentsDBAsync(new UpdateCourseByStudentsDBRequest()
                {
                    CourseCode = courseCode,
                    Students = students
                });
            }
            else
            {
                ThrowRpcException(CustomerRpcCode.NotUpdateForCourseStatus, "Not update for course status");
            }
            return false;
        }


        /// <summary>
        /// 学生加入用户组
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddStudentInCourseUserGroupAsync(AddStudentInCourseUserGroupRequest request)
        {
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroupCodeIsEmpty");
            }
            if (request.StudentCodes == null || !request.StudentCodes.Any())
            {
                ThrowRpcException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            var dbRequest = new AddStudentInCourseUserGroupDBRequest();
            dbRequest.UserGroupCode = request.UserGroupCode;
            dbRequest.StudentCodes = request.StudentCodes;
            return await _courseDBService.AddStudentInCourseUserGroupDBAsync(dbRequest);
        }

        /// <summary>
        /// 学习完成处理
        /// </summary>
        /// <returns></returns>
        public async Task<bool> StudyCompletedAsync(StudyCompletedRequest request)
        {
            if (string.IsNullOrEmpty(request.CourseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            if (string.IsNullOrWhiteSpace(request.StudentCode))
            {
                ThrowRpcException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            var dbRequest = new StudyCompletedDBRequest();
            dbRequest.CourseCode = request.CourseCode;
            dbRequest.StudentCode = request.StudentCode;
            return await _courseDBService.StudyCompletedDBAsync(dbRequest);
        }
    }
}
