using System.Reflection.Metadata;
using System.Linq;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingInterfaceLibrary.Result.Education;
using WingInterfaceLibrary.Rtc;
using WingInterfaceLibrary.Internal.Request;
using WingServerCommon.Log;
using WingInterfaceLibrary.Notifications.Consultation;
using WingInterfaceLibrary.DTO.RTC;
using System;
using WingInterfaceLibrary.Request.DBRequest;
using WingServerCommon.Config;
using WingInterfaceLibrary.DTO.Education;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.DTO.LiveRoom;
using Newtonsoft.Json;
using WingInterfaceLibrary.LiveConsultation;

namespace WingEducationService.Service
{
    /// <summary>
    /// 课程直播服务
    /// </summary>
    public partial class EducationService : JsonRpcService, IEducationService
    {
        /// <summary>
        /// 老师开始直播
        /// </summary>
        /// <param name="request">开始直播请求实体</param>
        /// <returns></returns>
        public async Task<InitiateLiveCourseResult> InitiateLiveCourseAsync(InitiateLiveCourseRequest request)
        {
            var teacherCode = await GetClientIdByTokenAsync(request.Token);
            //保存发起人
            var roomId = request.CourseCode;
            using (await _lockManager.Acquire(roomId))
            {
                // 发起人只能是老师
                var courseDetailInfo = CacheMaintenance.Instance.Get<ICourseInfosManager>().Where(c => c.Code == roomId)?.FirstOrDefault() ?? new CacheCourseInfoDTO();
                if (courseDetailInfo == null || courseDetailInfo.TeacherCode != teacherCode)
                {
                    ThrowRpcException(CustomerRpcCode.LiveTeacherNotFound, "Live teacher not find");
                }
                var roomRequest = new GetLiveRoomRequest()
                {
                    LiveRoomCode = roomId
                };
                var room = await _rtcService.GetLiveRoomAsync(roomRequest);
                if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
                {
                    Logger.WriteLineWarn($"WingEducationService initiate live course failed, course not existed, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course not existed");
                };
                var courseName = courseDetailInfo.Name;
                if (courseDetailInfo.Status == (int)CourseStatusEnum.NoApproval || courseDetailInfo.Status == (int)CourseStatusEnum.RejectApproval)
                {
                    Logger.WriteLineWarn($"WingEducationService [{courseName}] reservation record not reviewed, current status:{courseDetailInfo.Status}, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.CourseNotReviewed, "Course not reviewed");
                }
                if (courseDetailInfo.Status == (int)CourseStatusEnum.Started)
                {
                    Logger.WriteLineWarn($"WingEducationService [{courseName}] course in process, current status:{courseDetailInfo.Status}, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.CourseInProcess, "Course In Process");
                }
                if (courseDetailInfo.Status == (int)CourseStatusEnum.HasEnded || courseDetailInfo.Status == (int)CourseStatusEnum.Offline)
                {
                    Logger.WriteLineWarn($"WingEducationService [{courseName}] course ended, current status:{courseDetailInfo.Status}, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.CourseEnded, "Course ended");
                }
                //状态验证，老师/助教，一级学生是否在线, 建议优化，你本身获取房间的信息的时候，就应该所有数据都是新的
                var userCodes = new List<string>() { teacherCode };
                if (courseDetailInfo.Experts?.Count > 0)
                {
                    var expertUsers = courseDetailInfo.Experts.Select(c => c.Code).Distinct().ToList();
                    userCodes.AddRange(expertUsers);
                }
                if (courseDetailInfo.Assistants?.Count > 0)
                {
                    var assistantsUsers = courseDetailInfo.Assistants.Select(c => c.Code).Distinct().ToList();
                    userCodes.AddRange(assistantsUsers);
                }
                var memberRequest = new GetLiveMemberStatusInfosRequest()
                {
                    UserCodes = userCodes
                };
                var memberStautsInfo = await _rtcService.GetLiveMemberStatusInfosAsync(memberRequest);
                var roomInitiator = new LiveMemberStatusDTO();
                if (memberStautsInfo?.Count > 0)
                {
                    roomInitiator = memberStautsInfo.FirstOrDefault(c => c.Code == teacherCode) ?? new LiveMemberStatusDTO();
                }
                if (roomInitiator == null || roomInitiator.UserStatusType == UserStatusEnum.NotOnline || roomInitiator.UserStatusType == UserStatusEnum.IsBusy)
                {
                    Logger.WriteLineWarn($"WingEducationService [{courseName}] initiate live course failed, teacher not online, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.TeacherNotOnline, "Course Teacher not online");
                }
                if (memberStautsInfo.Count(x => x.UserStatusType == UserStatusEnum.Idle) < 1)
                {
                    Logger.WriteLineWarn($"WingEducationService [{courseName}] initiate live course failed, other members not online, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.StudentsNotOnline, "Course students not online");
                } 
               
                //修改预约记录状态
                var courseStatusRequest = new UpdateCourseStatusDBRequest()
                {
                    Code = roomId,
                    Status = CourseStatusEnum.Started
                };
                var updateResult = await _courseDBService.UpdateCourseStatusAsync(courseStatusRequest);
                if (!updateResult)
                {
                    Logger.WriteLineWarn($"WingEducationService [{room.Name}] update Status to db failed, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.CourseDBError, "Course DB error");
                }
               
                var initiateLive = new InitiateRequest()
                {
                    LiveRoomCode = roomId,
                    InitiatorCode = teacherCode
                };
                room = await _rtcService.InitiateAsync(initiateLive);
                var userSignRequest = new GetUserSignRequest
                {
                    UserId = teacherCode
                };
                var userSignResult = await _wingRtcService.GetUserSignAsync(userSignRequest);
                var result = new InitiateLiveCourseResult
                {
                    CourseCode = roomId,
                    RoomNo = room.RtcRoomId,
                    InitiatorCode = teacherCode,
                    AppId = room.SdkAppId,
                    UserSign = userSignResult.UserSign,
                    MemberLiveDatas = TransferLiveMemberToCourseMember(room.UserInfos, teacherCode, memberStautsInfo, courseDetailInfo.Experts, courseDetailInfo.Assistants),
                };

                return result;
            }
        }

        /// <summary>
        /// 学员加入房间
        /// </summary>
        /// <param name="request">加入课程请求实体</param>
        /// <returns></returns>
        public async Task<JoinLiveCourseResult> JoinLiveCourseAsync(JoinLiveCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //获取房间缓存
            var roomId = request.CourseCode;
            using (await _lockManager.Acquire(roomId))
            {
                var courseDetailInfo = CacheMaintenance.Instance.Get<ICourseInfosManager>().Where(c => c.Code == roomId)?.FirstOrDefault() ?? new CacheCourseInfoDTO();
                if (courseDetailInfo == null)
                {
                    ThrowRpcException(CustomerRpcCode.LiveTeacherNotFound, "Live teacher not find");
                }
                var courseName = courseDetailInfo.Name;
                if (courseDetailInfo.Status == (int)CourseStatusEnum.HasEnded || courseDetailInfo.Status == (int)CourseStatusEnum.Offline)
                {
                    Logger.WriteLineWarn($"WingEducationService [{courseName}] course ended, current status:{courseDetailInfo.Status}, roomId:{roomId}");
                    ThrowRpcException(CustomerRpcCode.CourseEnded, "Course ended");
                }
                //是否是付费课程
                var isNeedPay = courseDetailInfo.Price > 0 ? true : false;
                var roomRequest = new GetLiveRoomRequest()
                {
                    LiveRoomCode = roomId
                };
                var room = await _rtcService.GetLiveRoomAsync(roomRequest);
                if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
                {
                    ThrowRpcException(CustomerRpcCode.LiveCourseNotFound, "Live course not found");
                }
                var initiatorInfo = room.UserInfos.FirstOrDefault(c => c.Code == courseDetailInfo.TeacherCode) ?? new LiveMemberDTO();
                if (courseDetailInfo.Status != (int)CourseStatusEnum.Started || initiatorInfo.Status != LiveMemberStatus.Joined)
                {
                    ThrowRpcException(CustomerRpcCode.CourseNotInProcess, "Course not in process");
                }
                //验证是否收费
                var studentInfo = courseDetailInfo.Students.FirstOrDefault(c => c.Code == userCode) ?? new CacheStudentInfoDTO();
                if (!string.IsNullOrEmpty(studentInfo.Code))
                {
                    if (isNeedPay && !studentInfo.IsPay)
                    {
                        ThrowRpcException(CustomerRpcCode.StudentNotPay, "Student not pay");
                    }
                }
                //加入直播
                var acceptReq = new AcceptRequest()
                {
                    LiveRoomCode = roomId,
                    UserCode = userCode
                };
                var liveMember = await _rtcService.AcceptAsync(acceptReq);
                //获取签名
                var userSignRequest = new GetUserSignRequest
                {
                    UserId = userCode
                };
                var userSignResult = await _wingRtcService.GetUserSignAsync(userSignRequest);
                //状态验证，老师/助教，一级学生是否在线, 建议优化，你本身获取房间的信息的时候，就应该所有数据都是新的
                var userCodes = new List<string>() { courseDetailInfo.TeacherCode };
                if (courseDetailInfo.Experts?.Count > 0)
                {
                    var expertUsers = courseDetailInfo.Experts.Select(c => c.Code).Distinct().ToList();
                    userCodes.AddRange(expertUsers);
                }
                if (courseDetailInfo.Assistants?.Count > 0)
                {
                    var assistantsUsers = courseDetailInfo.Assistants.Select(c => c.Code).Distinct().ToList();
                    userCodes.AddRange(assistantsUsers);
                }
                var memberRequest = new GetLiveMemberStatusInfosRequest()
                {
                    UserCodes = userCodes
                };
                var memberStautsInfo = await _rtcService.GetLiveMemberStatusInfosAsync(memberRequest);
                var result = new JoinLiveCourseResult
                {
                    CourseCode = roomId,
                    UserCode = userCode,
                    UserSign = userSignResult.UserSign,
                    AppId = room.SdkAppId,
                    RoomNo = room.RtcRoomId,
                    MemberLiveDatas = TransferLiveMemberToCourseMember(room.UserInfos, courseDetailInfo.TeacherCode, memberStautsInfo, courseDetailInfo.Experts, courseDetailInfo.Assistants)
                };
                return result;
            }
        }

        /// <summary>
        /// Send HeartRate request in a diagnosis metting
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        public async Task<LiveHeartRateResult> HeartRateAsync(LiveHeartRateRequest request)
        {
            var userToken = await _authenticationService.GetTokenAsync(request);
            var roomRequest = new GetLiveRoomRequest()
            {
                LiveRoomCode = request.LiveCode
            };
            var room = await _rtcService.GetLiveRoomAsync(roomRequest);
            if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course not existed");
            }
            var courseDetailInfo = CacheMaintenance.Instance.Get<ICourseInfosManager>().Where(c => c.Code == request.LiveCode)?.FirstOrDefault() ?? new CacheCourseInfoDTO();
            if (courseDetailInfo == null)
            {
                ThrowRpcException(CustomerRpcCode.LiveTeacherNotFound, "Live teacher not find");
            }
            if (courseDetailInfo.Status == (int)CourseStatusEnum.NoApproval || courseDetailInfo.Status == (int)CourseStatusEnum.RejectApproval)
            {
                ThrowRpcException(CustomerRpcCode.ReservationNotReviewed, $"Reservation record not reviewed, current status:{courseDetailInfo.Status.ToString()}");
            }
            if (courseDetailInfo.Status != (int)CourseStatusEnum.Started)
            {
                ThrowRpcException(CustomerRpcCode.CourseEnded, $"Live Course Ended, current status:{courseDetailInfo.Status.ToString()}");
            }
            _liveHeartRateManager.AddOrUpdate(userToken.Code, userToken.ClientId, userToken.AccountName, request.LiveCode);
            return await Task.FromResult<LiveHeartRateResult>(new LiveHeartRateResult { LiveCode = request.LiveCode });
        }

        /// <summary>
        /// 结束课程直播
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        public async Task<LiveCourseBaseResult> LeaveLiveCourseAsync(JoinLiveCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //获取房间缓存
            var roomId = request.CourseCode;
            using (await _lockManager.Acquire(roomId))
            {
                var roomRequest = new GetLiveRoomRequest()
                {
                    LiveRoomCode = roomId
                };
                var room = await _rtcService.GetLiveRoomAsync(roomRequest);
                if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
                {
                    ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course not existed");
                }
                var liveMember = room.UserInfos?.FirstOrDefault(x => x.Code == userCode);
                if (liveMember == null || string.IsNullOrEmpty(liveMember.Code))
                {
                    ThrowRpcException(CustomerRpcCode.LivMemberNotFound, "Live member not found");
                }
                var isClosed = true;      
                var courseDetailInfo = CacheMaintenance.Instance.Get<ICourseInfosManager>().Where(c => c.Code == roomId)?.FirstOrDefault() ?? new CacheCourseInfoDTO();
                if (courseDetailInfo == null)
                {
                    ThrowRpcException(CustomerRpcCode.LiveTeacherNotFound, "Live teacher not find");
                }    
                var userCodes = new List<string>() { courseDetailInfo.TeacherCode };
                if (courseDetailInfo.Experts?.Count > 0)
                {
                    var expertUsers = courseDetailInfo.Experts.Select(c => c.Code).Distinct().ToList();
                    userCodes.AddRange(expertUsers);
                }
                if (courseDetailInfo.Assistants?.Count > 0)
                {
                    var assistantsUsers = courseDetailInfo.Assistants.Select(c => c.Code).Distinct().ToList();
                    userCodes.AddRange(assistantsUsers);
                }
                if (userCodes.Contains(userCode))
                {
                    //移除当前用户。因为当前用户肯定还没离线，看看其他的用户是否离线
                    userCodes.Remove(userCode);
                }
                if (room.UserInfos?.Count > 0)
                {
                    foreach(var item in room.UserInfos)
                    {
                        if (userCodes.Contains(item.Code) && (item.Status == LiveMemberStatus.Joined || item.Status == LiveMemberStatus.Accepted))
                        {
                            isClosed = isClosed & false;
                        }
                    }
                }
                var req = new LeaveRequest()
                {
                    LiveRoomCode = roomId,
                    UserCode = userCode,
                    Closed = isClosed
                };
                var res = await _rtcService.LeaveAsync(req);
                if (res && isClosed)
                {
                    //修改预约记录状态
                    var courseStatusRequest = new UpdateCourseStatusDBRequest()
                    {
                        Code = roomId,
                        Status = CourseStatusEnum.HasEnded
                    };
                    await _courseDBService.UpdateCourseStatusAsync(courseStatusRequest);
                }
                var result = new LiveCourseBaseResult() { CourseCode = roomId };
                return result;
            }
        }

        /// <summary>
        /// 设置开启或关闭静音状态
        /// </summary>
        /// <param name="request">设置开启或关闭静音状态的请求</param>
        /// <returns></returns>
        public async Task<LiveCourseBaseResult> SetMuteLiveCourseAsync(SetMuteLiveCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //获取房间缓存
            var roomId = request.CourseCode;
            var roomRequest = new GetLiveRoomRequest()
            {
                LiveRoomCode = roomId
            };
            var room = await _rtcService.GetLiveRoomAsync(roomRequest);
            if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course not existed");
            }
            if (room.Status != LiveRoomStatus.Connected && room.Status != LiveRoomStatus.Initiating)
            {
                ThrowRpcException(CustomerRpcCode.CourseNotInProcess, "Course Not InProcess");
            }
            else
            {
                var liveMember = room.UserInfos?.FirstOrDefault(x => x.Code == userCode);
                if (liveMember == null || string.IsNullOrEmpty(liveMember.Code))
                {
                    ThrowRpcException(CustomerRpcCode.LivMemberNotFound, "Live member not found");
                }
                var req = new ChangeMuteOpenedRequest()
                { 
                    LiveRoomCode = roomId,
                    UserCode = userCode, 
                    MuteOpened = request.IsMute
                };
                await _rtcService.ChangeMuteOpenedAsync(req);
            }
            var result = new LiveCourseBaseResult() { CourseCode = roomId };
            return result;
        }

        /// <summary>
        /// 开启或关闭视频
        /// </summary>
        /// <param name="request">开启或关闭视频请求</param>
        /// <returns></returns>       
        public async Task<LiveCourseBaseResult> SetVideoLiveCourseAsync(SetVideoLiveCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //获取房间缓存
            var roomId = request.CourseCode;
            var roomRequest = new GetLiveRoomRequest()
            {
                LiveRoomCode = roomId
            };
            var room = await _rtcService.GetLiveRoomAsync(roomRequest);
            if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course not existed");
            }
            if (room.Status != LiveRoomStatus.Connected && room.Status != LiveRoomStatus.Initiating)
            {
                ThrowRpcException(CustomerRpcCode.CourseNotInProcess, "Course Not InProcess");
            }
            else
            {
                var liveMember = room.UserInfos.FirstOrDefault(x => x.Code == userCode);
                if (liveMember == null || string.IsNullOrEmpty(liveMember.Code))
                {
                    ThrowRpcException(CustomerRpcCode.LivMemberNotFound, "Live member not found");
                }
                var req = new ChangeVideoOpenedRequest()
                {
                    LiveRoomCode = roomId,
                    UserCode = userCode, 
                    VideoOpened = request.IsOpened
                };
                await _rtcService.ChangeVideoOpenedAsync(req);
            }
            var result = new LiveCourseBaseResult() { CourseCode = roomId };
            return result;
        }

        /// <summary>
        /// 设置设备调参
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetControlParameterLiveCourseAsync(ControlParameterInCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //获取房间缓存
            var roomId = request.CourseCode;
            var roomRequest = new GetLiveRoomRequest()
            {
                LiveRoomCode = roomId
            };
            var room = await _rtcService.GetLiveRoomAsync(roomRequest);
            if (room == null || string.IsNullOrEmpty(room.LiveRoomCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course not existed");
            }
            var liveMember = room.UserInfos.FirstOrDefault(x => x.Code == userCode);
            if (liveMember == null || string.IsNullOrEmpty(liveMember.Code))
            {
                ThrowRpcException(CustomerRpcCode.LivMemberNotFound, "Live member not found");
            }
            var user = await _userDBService.FindUserByCodeAsync(userCode);
            //检查是否用用户使用参数调节中
            var isControllingMember = room.UserInfos.FirstOrDefault(x => x.IsControllingParameter);
            if (isControllingMember != null && isControllingMember.Code != userCode)
            {
                ThrowRpcException(CustomerRpcCode.ControllingParameterByOtherUser, "Controlling parameter by other");
            }
            var isControllingParameter = true;
            if (request.ControlType == ControlDeviceParameterEnum.End)
            {
                isControllingParameter = false;
            }
            if (liveMember.IsControllingParameter != isControllingParameter)
            {
                var req = new ChangeControllingStateRequest()
                {
                    LiveRoomCode = roomId,
                    UserCode = liveMember.Code,
                    DeviceCode = "",
                    IsControllingParameter = isControllingParameter
                };
                //todo 设备调参方法
                await _rtcService.ChangeControllingStateAsync(req);
            }
            return true;
        }

        #region private method
        /// <summary>
        /// 心跳检测进入房间
        /// </summary>
        /// <param name="roomId"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <returns></returns> 
        private async void HandleClientJoined(string roomId, string clientId, string name)
        {
            try
            {
                //获取房间缓存
                var roomRequest = new GetLiveRoomRequest()
                {
                    LiveRoomCode = roomId
                };
                var room = await _rtcService.GetLiveRoomAsync(roomRequest);
                var liveMember = room?.UserInfos.FirstOrDefault(x => x.Code == clientId);
                if (room != null && !string.IsNullOrEmpty(room.LiveRoomCode) && liveMember != null)
                {
                    if (liveMember.Status == LiveMemberStatus.Default || liveMember.Status == LiveMemberStatus.Accepted || liveMember.Status == LiveMemberStatus.Left)
                    {
                        var req = new HeartRateJoinRequest()
                        {
                            LiveRoomCode = roomId,
                            UserCode = liveMember.Code
                        };
                        await _rtcService.HeartRateJoinedAsync(req);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService HandleClientJoined err, name:{name}, clientId:{clientId}, roomId:{roomId}, ex:{ex}");
            }
        }

        /// <summary>
        /// 心跳检测网络异常
        /// </summary>
        /// <param name="roomId"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <returns></returns> 
        private async void HandleClientNetworkErr(string roomId, string clientId, string name)
        {
            try
            {
                //获取房间缓存
                var roomRequest = new GetLiveRoomRequest()
                {
                    LiveRoomCode = roomId
                };
                var room = await _rtcService.GetLiveRoomAsync(roomRequest);
                var liveMember = room?.UserInfos.FirstOrDefault(x => x.Code == clientId);
                if (room != null && !string.IsNullOrEmpty(room.LiveRoomCode) && liveMember != null)
                {
                    if (liveMember.Status == LiveMemberStatus.Joined)
                    {
                        var req = new NetworkErrorRequest()
                        {
                            LiveRoomCode = roomId,
                            UserCode = liveMember.Code
                        };
                        await _rtcService.NetworkErrorAsync(req);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService ClientNetworkErrAsync err, name:{name}, clientId:{clientId}, roomId:{roomId}, ex:{ex}");
            }
        }

        /// <summary>
        /// 心跳检测离线
        /// </summary>
        /// <param name="roomId"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <returns></returns> 
        private async void HandleClientLeave(string roomId, string clientId, string name)
        {
            try
            {
                //获取房间缓存
                var roomRequest = new GetLiveRoomRequest()
                {
                    LiveRoomCode = roomId
                };
                var room = await _rtcService.GetLiveRoomAsync(roomRequest);
                var liveMember = room?.UserInfos.FirstOrDefault(x => x.Code == clientId);
                if (room != null && !string.IsNullOrEmpty(room.LiveRoomCode) && liveMember != null)
                {
                    if (liveMember.Status == LiveMemberStatus.Joined)
                    {
                        var req = new HeartRateLeavedRequest()
                        {
                            LiveRoomCode = roomId,
                            UserCode = liveMember.Code
                        };
                        var res = await _rtcService.HeartRateLeavedAsync(req);
                        if (res)
                        {
                            bool isClosed = true;
                            // 发起人只能是老师
                            var courseDetailInfo = CacheMaintenance.Instance.Get<ICourseInfosManager>().Where(c => c.Code == roomId)?.FirstOrDefault() ?? new CacheCourseInfoDTO();
                            if (courseDetailInfo != null)
                            {
                                var userCodes = new List<string>() { courseDetailInfo.TeacherCode };
                                if (courseDetailInfo.Experts?.Count > 0)
                                {
                                    var expertUsers = courseDetailInfo.Experts.Select(c => c.Code).Distinct().ToList();
                                    userCodes.AddRange(expertUsers);
                                }
                                if (courseDetailInfo.Assistants?.Count > 0)
                                {
                                    var assistantsUsers = courseDetailInfo.Assistants.Select(c => c.Code).Distinct().ToList();
                                    userCodes.AddRange(assistantsUsers);
                                }
                                if (userCodes.Contains(clientId))
                                {
                                    //移除当前用户。因为当前用户肯定还没离线，看看其他的用户是否离线
                                    userCodes.Remove(clientId);
                                }
                                var memberRequest = new GetLiveMemberStatusInfosRequest()
                                {
                                    UserCodes = userCodes
                                };
                                var memberStautsInfo = await _rtcService.GetLiveMemberStatusInfosAsync(memberRequest);
                                if (memberStautsInfo?.Count > 0)
                                {
                                    foreach(var item in memberStautsInfo)
                                    {
                                        if (item.Status == LiveMemberStatus.Joined || item.Status == LiveMemberStatus.Accepted)
                                        {
                                            isClosed = isClosed & false;
                                        }
                                    }
                                }
                            }
                            if (isClosed) 
                            {
                                // 修改预约记录状态
                                var courseStatusRequest = new UpdateCourseStatusDBRequest()
                                {
                                    Code = roomId,
                                    Status = CourseStatusEnum.HasEnded
                                };
                                var updateResult = await _courseDBService.UpdateCourseStatusAsync(courseStatusRequest);
                              
                                var updateLiveRoomReq = new CloseRequest()
                                {
                                    LiveRoomCode = roomId,
                                    UserCode = liveMember.Code,
                                };
                                var closeResult = await _rtcService.CloseAsync(updateLiveRoomReq);
                                if (closeResult)
                                {
                                    Logger.WriteLineWarn($"LiveService HandleClientLeave CloseRoom, name:{name}, clientId:{clientId}, roomId:{roomId}");
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService HandleClientLeave err, name:{name}, clientId:{clientId}, roomId:{roomId}, ex:{ex}");
            }
        }
 
        /// <summary>
        /// room房间成员与课程成员状态枚举类型转换
        /// </summary>
        /// <param name="status"></param>
        /// <returns></returns>
        private WingInterfaceLibrary.LiveConsultation.LiveConsultationMemberStatus TransferMemberStatus(LiveMemberStatusEnum status)
        {
            var obj = Enum.ToObject(typeof(WingInterfaceLibrary.LiveConsultation.LiveConsultationMemberStatus), (int)status);
            var statusEnum = (WingInterfaceLibrary.LiveConsultation.LiveConsultationMemberStatus)obj;
            return statusEnum;
        }
        
        /// <summary>
        /// 学员加入房间
        /// </summary>
        /// <param name="request">加入课程请求实体</param>
        /// <returns></returns>
        private IList<LiveCourseMember> TransferLiveMemberToCourseMember(IList<LiveMemberDTO> members, string teacherCode, IList<LiveMemberStatusDTO> memberStatusList, IList<CacheStudentInfoDTO> experts, IList<CacheStudentInfoDTO> assistants)
        {
            IList<LiveCourseMember> memberList = new List<LiveCourseMember>();
            if (members?.Count > 0)
            {
                var initiator = members.FirstOrDefault(c => c.Code == teacherCode) ?? new LiveMemberDTO();
                foreach(var item in members)
                {
                    var memberStatusInfo = memberStatusList?.FirstOrDefault(c => c.Code == item.Code) ?? new LiveMemberStatusDTO();
                    var isExpertUser = (experts.Count(c => c.Code == item.Code) > 0) ? true : false;
                    var isAssistantUser = (assistants.Count(c => c.Code == item.Code) > 0) ? true : false;
                    var isInitiator = (item.Code == teacherCode) ? true : false;
                    var tempLiveData = isInitiator ? initiator.LiveData : null;
                    var courseMember = new LiveCourseMember()
                    {
                        LiveData = tempLiveData,
                        LoginSource = memberStatusInfo.LoginSource,
                        LoginServerUrl = memberStatusInfo.LoginServerUrl,
                        Status = item.Status,
                        VideoDeviceInfos = new List<VideoDeviceOutputInfo>(),
                        IsTeacher = isInitiator,
                        IsExpertUser = isExpertUser,
                        IsAssistantUser = isAssistantUser,
                        Mute = item.MuteOpened,
                        UserStatusType = memberStatusInfo.UserStatusType,
                        HeadImageToken = item.HeadImageToken,
                        MemberType = item.MemberType,
                        Name = item.Name,
                        Id = item.Code,
                        VideoOpend = item.VideoOpened,
                        IsControllingParameter = item.IsControllingParameter
                    };
                    memberList.Add(courseMember);
                }
            }
            return memberList;
        }
        #endregion 
    }
}