using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Result;
using System.Linq;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;
using System;
using WingInterfaceLibrary.Request.Lock;
using WingInterfaceLibrary.Internal.Interface;
using WingInterfaceLibrary.Rtc;
using WingEducationService.Common;

namespace WingEducationService.Service
{
    /// <summary>
    /// 教培服务
    /// </summary>
    public partial class EducationService : JsonRpcService, IEducationService
    {
        // private readonly int _sdkAppId;
        // private string _serverID;
        // private readonly int _connectionTimeout;
        private ICourseDBService _courseDBService;
        private IUserDBService _userDBService;
        private IAuthenticationService _authenticationService;
        private IEducationDBService _educationDBService;
        private LockManager _lockManager;
        private IWingRtcService _wingRtcService;
        private ILockService _lockService;
        private IDeviceInfoDBService _deviceDBService;
        private ILiveRoomDBService _liveRoomDBService;
        private IMasterInteractionCenterService _masterInteractionCenterService;
        private LiveHeartRateManager _liveHeartRateManager;
        private IPaymentDBService _paymentDBService;
        private IPaymentService _paymentService;
        private INotificationService _notificationService;
        private IRtcService _rtcService;

        /// <summary>
        /// 默认构造
        /// </summary>
        public EducationService()
        {
            // _serverID = ConfigurationManager.GetParammeter<StringParameter>("General", "ServerID").Value;
            // _connectionTimeout = ConfigurationManager.GetParammeter<IntParameter>("Live", "ConnectionTimeout").Value;
            // var sdkAppIdString = ConfigurationManager.GetParammeter<StringParameter>("Rtc", "RTCAppId").Value;
            // if (int.TryParse(sdkAppIdString, out int sdkAppId))
            // {
            //     _sdkAppId = sdkAppId;
            // }
            _liveHeartRateManager = new LiveHeartRateManager(HandleClientJoined, HandleClientNetworkErr, HandleClientLeave);
        }

        /// <summary>
        /// 初始化服务
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _courseDBService = GetProxy<ICourseDBService>();
            _userDBService = GetProxy<IUserDBService>();
            _liveRoomDBService = GetProxy<ILiveRoomDBService>();
            _educationDBService = GetProxy<IEducationDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _wingRtcService = GetProxy<IWingRtcService>();
            _rtcService = GetProxy<IRtcService>();
            _deviceDBService = GetProxy<IDeviceInfoDBService>();            
            _masterInteractionCenterService = GetProxy<IMasterInteractionCenterService>();
            _lockService = GetProxy<ILockService>();
            _paymentDBService = GetProxy<IPaymentDBService>();
            _paymentService = GetProxy<IPaymentService>();
            _notificationService = GetProxy<INotificationService>();
            var inProcessServiceSettings = ConfigurationManager.GetParammeter<StringParameter>("Services", "InProcess").Value;
            var remoteServiceSettings = ConfigurationManager.GetParammeter<StringParameter>("Services", "Remote").Value;
            if (!inProcessServiceSettings.Contains("LockService") && !remoteServiceSettings.Contains("LockService"))
            {
                _lockManager = new LockManager();
            }
            else
            {
                _lockManager = new LockManager(ApplyLock, ReleaseLock);
            }
        }

        #region Basic Method

        /// <summary>
        /// 通用异常处理方法
        /// </summary>
        /// <param name="customerRpcCodeEnum">异常标识码</param>
        /// <param name="msg">异常信息</param>
        /// <param name="internalMessage">内部异常信息，非必填</param>
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg, string internalMessage = null)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg, internalMessage);
        }

        /// <summary>
        /// Apply Lock
        /// </summary>
        /// <param name="lockKey">key</param>
        /// <returns>是否锁定</returns>
        private bool ApplyLock(string lockKey)
        {
            try
            {
                var result = _lockService.ApplyLockAsync(new ApplyLockRequest
                {
                    LockKey = lockKey
                }).Result;
                return result.IsSuccess;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveCourseService ApplyLock err, lock key:{lockKey}, ex:{ex}");
                return false;
            }
        }

        /// <summary>
        /// Release Lock
        /// </summary>
        /// <param name="lockKey">key</param>
        /// <returns></returns>
        private bool ReleaseLock(string lockUniqueCode)
        {
            try
            {
                var result = _lockService.ReleaseLockAsync(new ReleaseLockRequest
                {
                    LockUniqueCode = lockUniqueCode
                }).Result;
                return result.IsSuccess;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveCourseService ReleaseLock err, release key:{lockUniqueCode}, ex:{ex}");
                return false;
            }
        }

        /// <summary>
        /// 自定异常处理方法
        /// </summary>
        private void ThrowRpcException(CustomerRpcCode code, string message, string internalMessage = null)
        {
            base.ThrowRpcException((int)code, message, internalMessage ?? message);
        }

        /// <summary>
        /// 服务启动后注册执行
        /// </summary>
        public override async void LoadAfterRegister()
        {
            //todo 启动后判断是否有要开始直播的提醒
        }
        #endregion

        /// <summary>
        /// 获取标签列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>标签信息</returns>
        /// <errorCodes></errorCodes>
        public async Task<IList<CourseLabelDTO>> GetCourseLabelsAsync(QueryCourseLabelListRequest request)
        {
            var result = await _educationDBService.GetCourseLabelsDBAsync(request);
            return result;
        }

        /// <summary>
        /// 获取Token
        /// </summary>
        /// <param name="token"></param>
        /// <returns>用户id</returns>
        private async Task<string> GetClientIdByTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result.ClientId; //return User code 
        }

        /// <summary>
        /// 获取Token
        /// </summary>
        /// <param name="token"></param>
        /// <returns>用户id</returns>
        private async Task<TokenDTO> GetClientInfoByTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result;
        }

        /// <summary>
        /// 客户端根据创建人获取创建的用户组
        /// </summary> 
        /// <param name="request">创建人code</param>
        /// <returns></returns>
        public async Task<IList<UserGroupDTO>> GetUserGroupListsByCreatorCodeAsync(TokenRequest request)
        {
            var clientId = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(clientId))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCreatorCodeIsEmpty, "UserGroup creator code is empty");
            }
            var result = await _educationDBService.GetUserGroupListsDBByCreatorCodeAsync(clientId);
            return result;
        }

        /// <summary>
        /// 新增用户组
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> AddUserGroupAsync(UserGroupRequest request)
        {
            if (string.IsNullOrEmpty(request.Name))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupNameIsEmpty, "UserGroup name is empty");
            }
            if (request.MaxPeople <= 0)
            {
                ThrowRpcException(CustomerRpcCode.UserGroupMaxPeopleNotValid, "UserGroup max people not valid");
            }
            //用户权限验证
            if (string.IsNullOrEmpty(request.CreatorCode))
            {
                var tokenInfo = await GetClientInfoByTokenAsync(request.Token);
                if (tokenInfo == null || string.IsNullOrEmpty(tokenInfo.ClientId))
                {
                    ThrowRpcException(CustomerRpcCode.UserGroupCreatorCodeIsEmpty, "UserGroup creator code is empty");
                }
                request.CreatorCode = tokenInfo.ClientId;
            }
            else
            {
                var userInfo = CacheMaintenance.Instance.Get<IUserInfoManager>().Get(request.CreatorCode);
                if (string.IsNullOrEmpty(userInfo.Code))
                {
                    ThrowRpcException(CustomerRpcCode.UserGroupCreatorCodeIsEmpty, "UserGroup creator code is empty");
                }
            }
            var result = await _educationDBService.AddUserGroupDBAsync(request);
            return result;
        }

        /// <summary>
        /// 查询用户分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<StudentInfoDTO>> FindUserGroupStudentsPagesAsync(QueryStudentByGroupCodePageRequest request)
        {
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroup code is empty");
            }
            var list = new PageResult<StudentInfoDTO>();
            var dataInfo = await _educationDBService.FindUserGroupsByGroupCodeDBAsync(request.UserGroupCode, request.Keyword);
            if (dataInfo != null && dataInfo.Students?.Count > 0)
            {
                var studentList = dataInfo.Students;
                if (studentList?.Count > 0)
                {
                    var filterList = studentList.Skip(request.PageSize * (request.PageIndex - 1))?.Take(request.PageSize)?.ToList();
                    list.PageData = filterList;
                    list.TotalCount = studentList.Count;
                }
            }
            list.PageIndex = request.PageIndex;
            list.PageSize = request.PageSize;
            return list;
        }

        /// <summary>
        /// 用户组移除用户
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> RemoveStudentsByGroupCodeAsync(UserGroupFilterRequest request)
        {
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroup code is empty");
            }
            if (request.StudentCodes == null || request.StudentCodes.Count <= 0)
            {
                ThrowRpcException(CustomerRpcCode.UserGroupStudentsIsEmpty, "UserGroup students is empty");
            }
            var result = await _educationDBService.RemoveStudentsByGroupCodeDBAsync(request.UserGroupCode, request.StudentCodes);
            return result;
        }
    }
}